Package: illuminaio
Version: 0.54.0
Title: Parsing Illumina Microarray Output Files
Imports: base64
Suggests: RUnit, BiocGenerics, IlluminaDataTestFiles (>= 1.0.2),
        BiocStyle
Description: Tools for parsing Illumina's microarray output files, including IDAT.	
Authors@R: c(person("Keith", "Baggerly", role = "aut"),
             person("Henrik", "Bengtsson", role = "aut"),
             person(c("Kasper", "Daniel"), "Hansen", role = c("aut", "cre"),
                    email = "kasperdanielhansen@gmail.com"),
             person("Matt", "Ritchie", role = "aut"),
             person(c("Mike", "L."), "Smith", role = "aut"),
             person("Tim", "Triche Jr.", role = "ctb"))
License: GPL-2
URL: https://github.com/HenrikBengtsson/illuminaio
BugReports: https://github.com/HenrikBengtsson/illuminaio/issues
biocViews: Infrastructure, DataImport, Microarray, ProprietaryPlatforms
git_url: https://git.bioconductor.org/packages/illuminaio
git_branch: RELEASE_3_23
git_last_commit: 86b198e
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-28 22:42:02 UTC; biocbuild
Author: Keith Baggerly [aut],
  Henrik Bengtsson [aut],
  Kasper Daniel Hansen [aut, cre],
  Matt Ritchie [aut],
  Mike L. Smith [aut],
  Tim Triche Jr. [ctb]
Maintainer: Kasper Daniel Hansen <kasperdanielhansen@gmail.com>
