importFrom(utils, file_test)
importFrom(utils, packageDescription)
importFrom(utils, read.delim)
importFrom(utils, write.table)
importFrom(base64, decode)
useDynLib(illuminaio)
export(readBGX)
export(readBPM)
export(readIDAT)
useDynLib("illuminaio", .registration = TRUE, .fixes = "C_")

